platform.apiLevel = '2.3'

local disp=platform.window
local dispX=disp:width()
local dispY=disp:height()

function centerText(gc, text, midX, midY)
	gc:drawString(text, midX-gc:getStringWidth(text)/2, midY-gc:getStringHeight(text)/2, "top")
end

function centerTextWithRect(gc, text, midX, midY, rectColor)
	centerText(gc, text, midX, midY)
	gc:setColorRGB(rectColor)
	gc:drawRect(midX-gc:getStringWidth(text)/2, midY-gc:getStringHeight(text)/2, gc:getStringWidth(text), gc:getStringHeight(text))
end

function drawBoard(gc, sBX, sBY, sX, sY, bW, bH)--sB-sizeBoard, sX-startX, sY-startY, bW-boardWidth, bH-boardHeight
	local sizeRectX=bW/sBX
	local sizeRectY=bH/sBY
    for i=0, sBY do
        gc:drawLine(sX, sY+sizeRectY*i, sX+bW, sY+sizeRectY*i)
    end
	for i=0, sBX do
		gc:drawLine(sX+sizeRectX*i, sY, sX+sizeRectX*i, sY+bH)
	end
end

function fTriangle(gc, x1, y1, x2, y2, x3, y3)--f=fill
	local points={{x1, y1}, {x2, y2}, {x3, y3}}
	for i=1, 3 do
		for j=i,3 do
			if points[i][2]>points[j][2] then
				local tmp={points[i][1], points[i][2]}
				points[i][1]=points[j][1]
				points[i][2]=points[j][2]
				points[j][1]=tmp[1]
				points[j][2]=tmp[2]
			elseif points[i][2]==points[j][2] then
				if points[i][1]>points[j][1] then
					local tmp={points[i][1], points[i][2]}
					points[i][1]=points[j][1]
					points[i][2]=points[j][2]
					points[j][1]=tmp[1]
					points[j][2]=tmp[2]
				end
			end
		end
	end
	if points[1][2]==points[2][2] then--two points are smaller coords. increment y
		local xSlope1=(points[3][1]-points[1][1])/(points[3][2]-points[1][2])
		local xSlope2=(points[3][1]-points[2][1])/(points[3][2]-points[2][2])
		local startX=points[1][1]
		local endX=points[2][1]
		if endX<startX then
			local tmp=xSlope1
			xSlope1=xSlope2
			xSlope2=tmp
			tmp=startX
			startX=endX
			endX=tmp
		end
		for i=points[1][2], points[3][2] do
			gc:drawLine(startX, i, endX, i)
			startX=startX+xSlope1
			endX=endX+xSlope2
		end
	elseif points[2][2]==points[3][2] then--two points are larger coords. decrement y
		local xSlope1=-(points[1][1]-points[2][1])/(points[1][2]-points[2][2])
		local xSlope2=-(points[1][1]-points[3][1])/(points[1][2]-points[3][2])
		local startX=points[2][1]
		local endX=points[3][1]
		if endX<startX then
			local tmp=xSlope1
			xSlope1=xSlope2
			xSlope2=tmp
			tmp=startX
			startX=endX
			endX=tmp
		end
		for i=points[2][2], points[1][2], -1 do
			gc:drawLine(startX, i, endX, i)
			startX=startX+xSlope1
			endX=endX+xSlope2
		end
	else
		local xSlope=(points[3][2]-points[1][2])/(points[3][1]-points[1][1])
		local newX2=points[1][1]+1/xSlope*(points[2][2]-points[1][2])
		fTriangle(gc, points[2][1], points[2][2], newX2, points[2][2], points[3][1], points[3][2])
		fTriangle(gc, points[2][1], points[2][2], newX2, points[2][2], points[1][1], points[1][2])
	end
end

do shape=class()
	function shape.clone(s)
		local tmpShape=shape()
		tmpShape.type=s.type
		for i=1, #s.coords do
			tmpShape.coords[i]=s.coords[i]
			tmpShape.tmpCoords[i]=s.tmpCoords[i]
		end
		tmpShape.data=s.data
		tmpShape.rotAngle=s.rotAngle
		tmpShape.color=s.color
		tmpShape.color2=s.color2
		tmpShape.tx=s.tx
		tmpShape.ty=s.ty
		tmpShape.rotPtX=s.PtX
		tmpShape.rotPtY=s.PtY
		tmpShape.dilX=s.dilX
		tmpShape.dilY=s.dilY
		tmpShape.dil=s.dil
		return tmpShape
	end
	
	function shape:init()
		self.type=0
		self.coords={}
		self.tmpCoords={}
		self.data=0
		self.rotAngle=0
		self.color=0
		self.color2=0
		self.tx=0
		self.ty=0
		self.rotPtX=0
		self.rotPtY=0
		self.dilX=0
		self.dilY=0
		self.dil=1
	end
	function shape:moveCoords(s, x, y)
		local mag=math.sqrt((self.coords[s])^2+(self.coords[s+1])^2)
		if mag~=0 then
			local angle=math.asin((self.coords[s+1])/mag)+self.rotAngle
			self.coords[s]=mag*math.cos(angle)
			self.coords[s+1]=mag*math.sin(angle)
		else
			self.coords[s]=0
			self.coords[s+1]=0
		end
		self.coords[s]=self.coords[s]+x+self.tx
		self.coords[s+1]=self.coords[s+1]+y+self.ty
	end
	function shape:translate(x, y)
		self.tx=x
		self.ty=y
	end
	function shape:rotate(angle)
		self.rotAngle=angle%360
	end
	function shape:setRotPoint(x, y)
		
	end
	function shape:setDilatePoint(x, y)
		self.dilX=x
		self.dilY=y
	end
	function shape:dilate(dilation)
		self.dil=dilation
	end
	function shape:setColorRGB(hex)
		self.color=hex
	end
	function shape:set2ndColorRGB(hex)
		self.color2=hex
	end
	function shape:point(x, y)
		self.type=1
		self.coords={x, y}
	end
	function shape:line(x1, y1, x2, y2)
		self.type=2
		self.coords={x1, y1, x2, y2}
	end
	function shape:polyLine(points)
		self.type=3
		self.coords={}
		for i=1, #points do
			self.coords[i]=points[i]
		end
	end
	function shape:drawRect(x, y, width, height)
		self.type=4
		self.coords={x, y, width, height}
	end
	function shape:drawArc(x, y, width, height, startAngle, arcAngle)
		self.type=5
		self.coords={x, y, width, height, startAngle, arcAngle}
	end
	function shape:drawString(text, x, y)
		self.type=6
		self.coords={x,y}
		self.data=text
	end
	function shape:image(image, x, y)
		self.type=7
		shape.coords={x,y}
		self.data=image
	end
	function shape:drawRoundRect(x, y, width, height, radius)
		self.type=8
		self.coords={x, y, width, height, radius}
	end
	function shape:fillPolygon(points)
		self.type=9
		self.coords={}
		for i=1, #points do
			self.coords[i]=points[i]
		end
	end
	function shape:fillRect(x, y, width, height)
		self.type=10
		self.coords={x, y, width, height}
	end
	function shape:fillArc(x, y, width, height, startAngle, arcAngle)
		self.type=11
		self.coords={x, y, width, height, startAngle, arcAngle}
	end
	function shape:fillRoundRect(x, y, width, height, radius)
		self.type=12
		self.coords={x, y, width, height, radius}
	end
	function shape:centeredString(text, x, y)
		self.type=13
		self.coords={x, y}
		self.data=text
	end
	function shape:gradientRect(x, y, width, height, d)
		self.type=14
		self.coords={x, y, width, heigh, d}
	end
	function shape:fillCircle(x, y, r)
		self.type=15
		self.coords={x, y, r}
	end
	function shape:gradientCircle(x, y, r)
		self.type=16
		self.coords={x, y, r}
	end
	function shape:draw(gc, x,y)
		gc:setColorRGB(self.color)
		for i=1, #self.coords do
			self.tmpCoords[i]=self.coords[i]
		end
		if self.type==1 then
			self:moveCoords(1, x, y)
			gc:drawPoint(self.coords[1], self.coords[2])
		elseif self.type==2 then
			self:moveCoords(1, x, y)
			self:moveCoords(3, x, y)
			gc:drawLine(self.coords[1], self.coords[2], self.coords[3], self.coords[4])
		elseif self.type==3 then
			for i=1, #self.coords, 2 do
				self:moveCoords(i, x, y)
			end
			gc:drawPolyLine(self.coords)
		elseif self.type==4 then
			self:moveCoords(1, x, y)
			gc:drawRect(self.coords[1], self.coords[2], self.coords[3], self.coords[4])
		elseif self.type==5 then
			self.coords[1]=self.coords[1]+self.tx+x
			self.coords[2]=self.coords[2]+self.ty+y
			self.coords[5]=(self.coords[5]+self.rotAngle)%360
			if self.coords[5]>=360 then
				self.coords[5]=self.coords[5]-360
			end
			if self.coords[6]>=360 then
				self.coords[6]=self.coords[6]-360
			end
			gc:drawArc(self.coords[1], self.coords[2], self.coords[3], self.coords[4], self.coords[5], self.coords[6])
		elseif self.type==6 then
			self:moveCoords(1, x, y)
			gc:drawString(self.data, self.coords[1], self.coords[2])
		elseif self.type==7 then
			self:moveCoords(1, x, y)
			gc:drawImage(self.data, self.coords[1], self.coords[2])
		elseif self.type==8 then
			self:moveCoords(1, x, y)
			gc:drawRoundRect(self.coords[1], self.coords[2], self.coords[3], self.coords[4], self.coords[5])
		elseif self.type==9 then
			for i=1, #self.coords, 2 do
				self:moveCoords(i, x, y)
			end
			gc:fillPolygon(self.coords)
		elseif self.type==10 then
			self:moveCoords(1, x, y)
			gc:fillRect(self.coords[1], self.coords[2], self.coords[3], self.coords[4])
		elseif self.type==11 then
			self.coords[1]=self.coords[1]+self.tx+x
			self.coords[2]=self.coords[2]+self.ty+y
			self.coords[5]=(self.coords[5]+self.rotAngle)%360
			if self.coords[5]>=360 then
				self.coords[5]=self.coords[5]%360
			end
			if self.coords[6]>=360 then
				self.coords[6]=self.coords[6]%360
			end
			gc:fillArc(self.coords[1], self.coords[2], self.coords[3], self.coords[4], self.coords[5], self.coords[6])
		elseif self.type==12 then
			self:moveCoords(1, x, y)
			gc:fillRoundRect(self.coords[1], self.coords[2], self.coords[3], self.coords[4], self.coords[5])
		elseif self.type==13 then
			self:moveCoords(1, x, y)
			centerText(gc, self.data, self.coords[1], self.coords[2])
		elseif self.type==14 then
			self:moveCoords(1, x, y)
			local tmpcolors1={math.floor(self.color/0x10000), math.floor((self.color%0x10000)/0x100), self.color%0x100}
			local tmpcolors2={math.floor(self.color2/0x10000), math.floor((self.color2%0x10000)/0x100), self.color%0x100}
			gc:fillGradientRect(self.coords[1], self.coords[2], self.coords[3], self.coords[4], tmpcolors1, tmpcolors2)
		elseif self.type==15 then
			self:moveCoords(1, x, y)
			gc:fillCircle(self.coords[1], self.coords[2], self.coords[3])
		elseif self.type==16 then
			self.coords[1]=self.coords[1]+self.tx+x
			self.coords[2]=self.coords[2]+self.ty+y
			local tmpcolors1={math.floor(self.color/0x10000), math.floor((self.color%0x10000)/0x100), self.color%0x100}
            local tmpcolors2={math.floor(self.color2/0x10000), math.floor((self.color2%0x10000)/0x100), self.color%0x100}
			gc:fillGradientCircle(self.coords[1], self.coords[2], self.coords[3], tmpcolors1, tmpcolors2)
		end
		for i=1, #self.coords do
			self.coords[i]=self.tmpCoords[i]
		end
	end
end

do glib=class()
	function glib:init(width, height)
		self.width=width
		self.height=height
		self.Shapes={}
		self.rotAngle=0
	end
	function glib:addShape(s, x, y, rotAngle)
		self.Shapes[#self.Shapes+1]=shape.clone(s)
		self.Shapes[#self.Shapes]:translate(x, y)
	end
	function glib:rotate(angle)
		self.rotAngle=angle%360
	end
	function glib:changeRot(angle)
		self.rotAngle=(self.rotAngle+angle)%360
	end
	function glib:draw(gc, x, y, mode)
		local changeX=0
		local changeY=0
		if mode~=nil then
			if mode=="middle" then
				changeX=self.width/2
				changeY=self.height/2
			elseif mode=="midtop" then
				changeX=self.width/2
			elseif mode=="midbottom" then
				changeX=self.width/2
				changeY=self.height
			end
		end
		gc:clipRect("set", x-changeX, y-changeY, self.width, self.height)
		for i=1, #self.Shapes do
			local tmp=self.Shapes[i].rotAngle
			self.Shapes[i]:rotate(self.Shapes[i].rotAngle+self.rotAngle)
			self.Shapes[i]:draw(gc, x, y)
			self.Shapes[i]:rotate(tmp)
		end
		gc:clipRect("reset")
	end
end