--include "menu.lua"
--include "errorHandler.lua"

function sign(x, zero)
	if zero~=nil and zero==true and x==0 then
		return 0
	elseif x==math.abs(x) then
		return 1
	else
		return -1
	end
end

function smpArrCmpr(arr1, arr2)
	if #arr1~=#arr2 then
		return false
	end
	
	for i=1, #arr1 do
		if arr1[i]~=arr2[i] then
			return false
		end
	end
	
	return true
end

function drawStrings(gc, strings)
	local height=gc:getStringHeight(strings[1])
	for i=1, #strings do
		gc:drawString(strings[i], dispX/2-gc:getStringWidth(strings[i])/2, dispY/2-height*(#strings/2-i+1))
	end
end

local settList={ 
	spd={n="Snake speed: ", ant=8},
	mapSize={n="Map size: ", ant=20}
}

do game=process()
	local snake={}
	local apple={}
	local mapSize={x={20, 260}, y={15, 195}}
	local direction={0,0}
	local adder=0
	local adderNum=3
	local score=0
	local highscore=0
	local minLeeway=30
	
	local colors={border=0xFF7F00, back=0x00FFFF, snake=0xFFFF00, app=0xFF0000}
	
	do die=process()
		function die:Open()
			if score>highscore then
				highscore=score
				document.markChanged()
			end
		end
		
		function die:paint(gc)
			gc:setColorRGB(0x000000)
			gc:fillRect(0,0,dispX, dispY)
			
			local drawData={
				"You died!",
				"Your score is "..score,
				"Your highscore is "..highscore,
				"Press enter to play again",
				"Press escape to exit"
			}
			
			gc:setColorRGB(0xFFFFFF)
			gc:setFont("serif", "r", 18)
			drawStrings(gc, drawData)
		end
		
		function die:enterKey()
			PullProcess()
			PushProcess(game)
		end
		
		function die:escapeKey()
			PullProcess()
		end
	end
	
	
	function setApple()
		apple={math.random(mapSize.x[1]), math.random(mapSize.y[1])}
	end
	
	function eatApple()
		if snake[1][1]==apple[1] and snake[1][2]==apple[2] then
			if #snake==1 then
				snake[2]={snake[1][1], snake[1][2]}
			end
			adder=adder+adderNum
			score=score+adderNum
			setApple()
		end
	end
	
	function reset()
		snake={{2, 2}}
		score=0
		direction={0,0}
		adder=0
		local ytmp=math.floor((dispY-minLeeway)/(settList.mapSize.ant*3/4))*(settList.mapSize.ant*3/4)
		mapSize={x={settList.mapSize.ant, ytmp*4/3}, y={settList.mapSize.ant*3/4, ytmp}}
		setApple()
		collectgarbage()
	end
	
	function SF()
		return mapSize.x[2]/mapSize.x[1]
	end
	
	function getXY(x, y)
		return {(dispX-mapSize.x[2])/2+((x-1)*SF())+1, (dispY-mapSize.y[2])/2+((y-1)*SF())+1}
	end
	
	function drawBit(gc, c1, c2)
		if c1[1]==c2[1] then
			for i=c1[2], c2[2], sign(c2[2]-c1[2]) do
				local coords=getXY(c1[1], i)
				gc:fillRect(coords[1], coords[2], SF()-2, SF()-2)
			end
		elseif c1[2]==c2[2] then
			for i=c1[1], c2[1], sign(c2[1]-c1[1]) do
				local coords=getXY(i, c1[2])
				gc:fillRect(coords[1], coords[2], SF()-2, SF()-2)
			end
		end
	end
	
	function keyToDir(key)
		local dir={}
		if key=="up" or key=="8" then
			dir={0,-1}
		elseif key=="right" or key=="6" then
			dir={1,0}
		elseif key=="left" or key=="4" then
			dir={-1,0}
		elseif key=="down" or key=="2" then
			dir={0,1}
		end
		return dir
	end
	
	function moveSnake()
		snake[1][1]=snake[1][1]+direction[1]
		snake[1][2]=snake[1][2]+direction[2]
		if #snake>1 then
			local tamp={0,0}
			if not smpArrCmpr(direction, tamp) then
				if adder<1 then
					snake[#snake][1]=snake[#snake][1]+sign(snake[#snake-1][1]-snake[#snake][1], true)
					snake[#snake][2]=snake[#snake][2]+sign(snake[#snake-1][2]-snake[#snake][2], true)
				else
					adder=adder-1
				end
			end
			if smpArrCmpr(snake[#snake-1], snake[#snake]) then
				table.remove(snake)
			end
		end
	end
	
	function updateDir(key)
		local dir=keyToDir(key)
		if (#snake>2 and (smpArrCmpr(snake[1], snake[2]))) then
			if not (smpArrCmpr(dir, {sign(snake[3][1]-snake[1][1], true), sign(snake[3][2]-snake[1][2], true)})) then
				table.remove(snake, 2)
			else
				return
			end
		end
		if (not (smpArrCmpr(dir, direction) or (#snake>1 and smpArrCmpr(dir, {sign(snake[2][1]-snake[1][1], true), sign(snake[2][2]-snake[1][2], true)})))) then
			local tmp={snake[1][1], snake[1][2]}
			table.insert(snake, 1, tmp)
			direction={dir[1], dir[2]}
		end
	end
	
	function game:paint(gc)
		gc:setColorRGB(colors.border)
		gc:fillRect(0,0,dispX, dispY)
		gc:setColorRGB(colors.back)
		gc:fillRect((dispX-mapSize.x[2])/2,(dispY-mapSize.y[2])/2, mapSize.x[2], mapSize.y[2])
		
		gc:setColorRGB(colors.snake)
		for i=1, #snake-1 do
			drawBit(gc, snake[i], snake[i+1])
		end
		
		if #snake==1 then
			drawBit(gc, snake[1], snake[1])
		end
		
		gc:setColorRGB(colors.app)
		if apple~=nil then
			drawBit(gc, apple, apple)
		end
		
		gc:setColorRGB(0xFFFFFF)
		gc:setFont("serif", "r", 11)
		gc:drawString("score: "..score, 0, 0)
	end
	
	function checkCollision(coords, startNum, endNum)
		for i=startNum, endNum-1 do
			if snake[i][1]==snake[i+1][1] then--vertical
				if coords[1]==snake[i][1] then
					if math.abs(snake[i][2]-snake[i+1][2])==math.abs(snake[i][2]-coords[2])+math.abs(snake[i+1][2]-coords[2]) then
						return true
					end
				end
			elseif snake[i][2]==snake[i+1][2] then--vertical
				if coords[2]==snake[i][2] then
					if math.abs(snake[i][1]-snake[i+1][1])==math.abs(snake[i][1]-coords[1])+math.abs(snake[i+1][1]-coords[1]) then
						return true
					end
				end
			end
		end
		return false
	end
	
	function game:timer()
		moveSnake()
		eatApple()
		
		if snake[1][1]<1 or snake[1][2]<1 or snake[1][1]>mapSize.x[1] or snake[1][2]>mapSize.y[1] or checkCollision(snake[1], 2, #snake) then
			PullProcess()
			menu:deactivate()
			PushProcess(die)
		end
	end
	
	function game:arrowKey(key)
		updateDir(key)
	end
	
	function game:charIn(c)
		if c=="8" or c=="6" or c=="4" or c=="2" then
			updateDir(c)
		end
	end
	
	function game:Open()
		reset()
		timerInvalidate=true
		timer.start(1/settList.spd.ant)
	end
	
	function game:activate()
	end
	
	function game:Close()
		timerInvalidate=false
		timer.stop()
	end
	
	function game:escapeKey()
		PullProcess()
	end
	
	
	function game:editSettings(item, key)
		if item.n==settList.spd.n then
			if key=="left" then
				if item.ant>1 then
					item.ant=item.ant-0.5
				end
			else
				item.ant=item.ant+0.5
			end
			settList.spd.ant=item.ant
		elseif item.n==settList.mapSize.n then
			if key=="left" then
				if item.ant>8 then
					item.ant=item.ant-4
				end
			else
				item.ant=item.ant+4
			end
			settList.mapSize.ant=item.ant
		end
		return item.ant
	end
	
	
	function game:save()
		return {high=highscore}
	end
	
	function game:restore(arr)
		if arr~=nil and arr.high~=nil then
			highscore=arr.high
		end
	end
end

function editSettings(item, key)
	return game:editSettings(item, key)
end

function on.save()
	return game:save()
end

function on.restore(arr)
	game:restore(arr)
end

function on.construction()
	settings:setFunc(editSettings)
	settings:addSettings(settList)
	menu:add("play", game)
	menu:add("settings", settings)
	PushProcess(menu)
end